package cc.fascinated.bat.command.impl.botadmin.premium;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "remove", description = "Remove premium from a guild")
public class RemoveSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public RemoveSubCommand(GuildService guildService) {
        this.guildService = guildService;
        super.addOption(OptionType.STRING, "guild", "The guild id to set as premium", true);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        OptionMapping guildOption = interaction.getOption("guild");
        if (guildOption == null) {
            interaction.reply("Please provide a guild id").queue();
            return;
        }
        String guildId = guildOption.getAsString();
        BatGuild batGuild = guildService.getGuild(guildId);
        if (batGuild == null) {
            interaction.reply("The guild with the id %s does not exist".formatted(guildId)).queue();
            return;
        }
        BatGuild.Premium premium = batGuild.getPremium();
        if (!premium.hasPremium()) {
            interaction.reply("The guild does not have premium").queue();
            return;
        }

        premium.removePremium();
        guildService.saveGuild(batGuild);
        interaction.reply("The guild **%s** has had its premium removed".formatted(guild.getName())).queue();
    }
}
