package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.TextChannelUtils;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.entities.channel.unions.GuildChannelUnion;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:channel.sub")
@CommandInfo(name = "channel", description = "Sets the birthday notification channel", requiredPermissions = Permission.MANAGE_SERVER)
public class ChannelSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public ChannelSubCommand(GuildService guildService) {
        super.addOption(OptionType.CHANNEL, "channel", "The channel birthdays will be sent in", false);
        this.guildService = guildService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        BirthdayProfile profile = guild.getProfile(BirthdayProfile.class);
        OptionMapping option = interaction.getOption("channel");
        if (option == null) {
            if (!TextChannelUtils.isValidChannel(profile.getChannelId())) {
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("There is no channel set for birthday notifications. Please provide a channel to set the birthday channel to")
                        .build()).queue();
                return;
            }
            interaction.replyEmbeds(EmbedUtils.genericEmbed()
                    .setDescription("The current birthday channel is %s".formatted(TextChannelUtils.getChannelMention(profile.getChannelId())))
                    .build()).queue();
            return;
        }

        GuildChannelUnion targetChannel = option.getAsChannel();
        if (targetChannel.getType() != ChannelType.TEXT) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid channel type, please provide a text channel")
                    .build()).queue();
            return;
        }

        profile.setChannelId(targetChannel.getId());
        guildService.saveGuild(guild);

        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully set the birthday channel to %s".formatted(targetChannel.asTextChannel().getAsMention()))
                .build()).queue();
    }
}
