package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:message.sub")
@CommandInfo(name = "message", description = "Changes the message that is sent when it is a user's birthday", requiredPermissions = Permission.MANAGE_SERVER)
public class MessageSubCommand extends BatSubCommand {
    private static final SimpleDateFormat FORMATTER = new SimpleDateFormat("dd/MM/yyyy");
    private final GuildService guildService;

    @Autowired
    public MessageSubCommand(GuildService guildService) {
        super.addOption(OptionType.STRING, "message", "The message that is sent. (Placeholders: {user}, {age})", true);
        this.guildService = guildService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        BirthdayProfile profile = guild.getProfile(BirthdayProfile.class);

        OptionMapping messageOption = interaction.getOption("message");
        if (messageOption == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You must provide a message")
                    .build()).queue();
            return;
        }

        String message = messageOption.getAsString();
        if (message.length() > 2000) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The message must be less than 2000 characters")
                    .build()).queue();
            return;
        }
        if (!message.contains("{user}") || !message.contains("{age}")) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The message must contain the placeholders {user} and {age}")
                    .build()).queue();
            return;
        }

        profile.setMessage(message);
        guildService.saveGuild(guild);

        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("You have updated the birthday message!\n\n**Message:** %s".formatted(profile.getBirthdayMessage(user.getDiscordUser())))
                .build()).queue();
    }

    /**
     * Parses a birthday from the string
     *
     * @param birthday the date to parse
     * @return the birthday
     */
    private Date parseBirthday(String birthday) {
        try {
            return FORMATTER.parse(birthday);
        } catch (ParseException ignored) {
        }
        return null;
    }
}
