package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.scoresaber.profile.UserScoreSaberProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.service.ScoreSaberService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber:link.sub")
@CommandInfo(name = "link", description = "Links your ScoreSaber profile")
public class LinkSubCommand extends BatSubCommand {
    private final ScoreSaberService scoreSaberService;
    private final UserService userService;

    @Autowired
    public LinkSubCommand(@NonNull ScoreSaberService scoreSaberService, @NonNull UserService userService) {
        super.addOption(OptionType.STRING, "link", "Link your ScoreSaber profile", true);
        this.scoreSaberService = scoreSaberService;
        this.userService = userService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        OptionMapping option = interaction.getOption("link");
        if (option == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Please provide a ScoreSaber profile link")
                    .build()).queue();
            return;
        }

        String link = option.getAsString();
        if (!link.contains("scoresaber.com/u/")) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid ScoreSaber profile link")
                    .build()).queue();
            return;
        }

        String id = link.split("scoresaber.com/u/")[1];
        if (id.contains("/")) {
            id = id.split("/")[0];
        }

        ScoreSaberAccountToken account = scoreSaberService.getAccount(id);
        if (account == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid ScoreSaber profile link")
                    .build()).queue();
            return;
        }

        user.getProfile(UserScoreSaberProfile.class).setSteamId(id);
        userService.saveUser(user);
        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully linked your [ScoreSaber](%s) profile".formatted("https://scoresaber.com/u/%s".formatted(id)))
                .build()).queue();
    }
}
