package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.spotify.profile.SpotifyProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "unlink", description = "Unlink your Spotify account")
public class UnlinkSubCommand extends BatSubCommand implements EventListener {
    private final UserService userService;

    @Autowired
    public UnlinkSubCommand(@NonNull UserService userService) {
        this.userService = userService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        SpotifyProfile profile = user.getProfile(SpotifyProfile.class);
        if (!profile.hasLinkedAccount()) {
            interaction.replyEmbeds(EmbedUtils.genericEmbed()
                            .setDescription("You do not have a linked Spotify account.")
                            .build())
                    .queue();
            return;
        }

        profile.reset();
        userService.saveUser(user);
        interaction.replyEmbeds(EmbedUtils.genericEmbed()
                        .setDescription("Successfully unlinked your Spotify account.")
                        .build())
                .queue();
    }
}
